library(tidyverse)
library(rvest)
library(openxlsx)

base_url_str = 
  "https://www.maxqda.de/support/forum/viewforum.php?f=23
  https://www.maxqda.de/support/forum/viewforum.php?f=23&start=50
  https://www.maxqda.de/support/forum/viewforum.php?f=23&start=100
  https://www.maxqda.de/support/forum/viewforum.php?f=23&start=150
  https://www.maxqda.de/support/forum/viewforum.php?f=23&start=200
  https://www.maxqda.de/support/forum/viewforum.php?f=23&start=250
  https://www.maxqda.de/support/forum/viewforum.php?f=23&start=300"

base_urls <- str_split(base_url_str, "\\s+") %>% unlist()
# base_urls
root_url <- "https://www.maxqda.de/support/forum"

base_pages <- 
  tibble(baseurl=base_urls) %>%
  rowwise() %>%
  mutate(doc=list(read_html(baseurl)))

# base_pages

thread_pages <- base_pages %>%
  rowwise() %>%
  mutate(
    thread_url = list(doc %>% html_nodes("ul.topiclist.topics li.row dt a.topictitle") %>% html_attr("href")),
    thread_title = list(doc %>% html_nodes("ul.topiclist.topics li.row dt a.topictitle") %>% html_text()),
    antworten = list(doc %>% html_nodes("ul.topiclist.topics li.row dd.posts") %>% html_text()),
    views = list(doc %>% html_nodes("ul.topiclist.topics li.row dd.views") %>% html_text())
  )  %>%
  select(-doc) %>%
  unnest(c(thread_url:views)) %>%
  rowwise() %>%
  mutate(
    thread_url=paste0(root_url,str_remove(thread_url, "^\\.")),
    doc=list(read_html(thread_url)), 
  )

# thread_pages

clean_html_text <- function(nodes) {
  lapply(nodes,
         function(node)
         { node %>%
             xml_contents() %>%
             paste(collapse="\n") %>%
             str_remove_all("<.*?>") %>%
             str_replace_all("\n+","\n\n")
         }) %>%
    unlist()
  }

postings <- thread_pages %>% 
  rowwise() %>% 
  mutate(
    titel = list(doc %>% html_nodes("div.postbody h2") %>% html_text()),
    datum = list(doc %>% html_nodes("div.postbody p.author") %>% html_text()),
    autor = list(doc %>% html_nodes("dl.postprofile dt") %>% html_text()),
    first = list(!is.na(doc %>% html_nodes("div.postbody h2") %>%  html_attr("class"))),
    content = list(doc %>% html_nodes("div.postbody div.content") %>% clean_html_text())
  ) %>%
  select(-doc) %>%
  unnest(c(titel:content)) %>%
  mutate(
    content_size =str_length(content)
  )

# postings

if (Sys.info()["sysname"]=="Windows") {
  postings$datum <- postings$datum %>% str_replace("Mär","Mrz")
}

forum_postings <- 
  postings %>%
  mutate(
    antworten = antworten %>% str_extract("^\\d+") %>% as.numeric(),
    views =  views %>% str_extract("^\\d+") %>% as.numeric(),
    autor = str_trim(autor),
    datum_zeit = datum %>% strptime(format="%d %b %Y, %H:%M", tz="CET"),
    datum = datum_zeit %>% strftime("%Y-%m-%d"),
    uhrzeit = datum_zeit %>% strftime("%H:%M:%S"),
    timestamp = datum_zeit %>% strftime("%Y-%m-%d %H:%M:%S"),
    jahr = datum_zeit %>% strftime("%Y") %>% as.numeric(),
    monat = datum_zeit %>% strftime("%m %B"),
    jahr_monat = paste(jahr,monat, sep="_"),
    tag = datum_zeit %>% strftime("%d") %>% as.numeric(),
    wtag = datum_zeit %>% strftime("%u %A"),
    stunde=datum_zeit %>% strftime("%H") %>% as.numeric(),
    size = str_length(content),
    maxqda_ver = content %>% str_extract(regex("(?<=^Version: ).*$", multiline = TRUE)),
    os_ver = content %>% str_extract(regex("(?<=^System: ).*$", multiline = TRUE))) %>%
  group_by(thread_url) %>%
  mutate(thread_zeit = first(timestamp),
         thread_datum = first(datum),
         first_poster = first(autor)) %>%
  ungroup() %>%
  mutate(
    Dokumentgruppe = paste(strftime(thread_zeit, format="%Y-%m\\%d"), 
                           thread_title,
                           sep = "-") %>% 
      str_sub(1, 63),
    Dokumentname = paste(timestamp, autor, sep = "_"), 
    typ = if_else(autor==first_poster, "Frage", "Antwort")
  )  %>%
  arrange(Dokumentgruppe,Dokumentname) %>% 
  select(Dokumentgruppe,Dokumentname,
         thread_datum, thread_zeit,
         thread_url,thread_title,
         antworten, views,
         datum,datum_zeit, 
         uhrzeit, timestamp,size, 
         jahr,monat, jahr_monat, tag, wtag, stunde, 
         os_ver, maxqda_ver, typ,
         autor, titel ,first_poster, first, content)

forum_postings %>% 
  select(Dokumentgruppe,Dokumentname,
         thread_url,thread_title,thread_datum, antworten, views,
         datum,uhrzeit, timestamp,
         jahr, monat, jahr_monat, tag, wtag, stunde,
         size, os_ver, maxqda_ver,
         titel,autor, first, typ, content) %>%
  arrange(Dokumentgruppe, Dokumentname) %>%
  write.xlsx(file = "MAXQDA_mit_R_V2.xlsx",
             asTable = TRUE, 
             colWidth = "auto")

forum_postings_1 <- 
  forum_postings %>%
  mutate(
    thread_datum_2 = thread_datum,
    across(.cols=c(thread_datum,datum:autor),  
           ~paste(names(forum_postings[cur_column()]),.x,sep="\\"),
           .names = "{col}_c")) %>%   
  arrange(Dokumentgruppe, Dokumentname) %>% 
  select(Dokumentgruppe, Dokumentname,
         thread_url,thread_title, thread_datum_2,  first_poster, jahr, monat, tag, stunde, antworten, views, 
         timestamp, thread_datum_c, datum_c, jahr_c, monat_c, jahr_monat_c, tag_c, wtag_c, stunde_c,
         size, os_ver_c,  maxqda_ver_c,
         autor, autor_c, titel, typ_c, content) %>%
  mutate(content = content %>% str_replace_all("\n","<br>"))

fn <- "MAXQDA_mit_R_V1.html"
if (file.exists(fn)) 
  file.remove(fn)
cn<- file(fn, open = "a")

write(
  '<!DOCTYPE html>
      <html lang="de">
      <head>
      <meta charset="UTF-8"/>
      </head>
      <body>',
  file=cn, append = TRUE)
forum_postings_1 %>%
  group_by(Dokumentgruppe) %>%
  group_split() %>%
  walk(function(thread){
    write(paste("<br>#TEXT ",thread$Dokumentgruppe[1]),file=cn, append = T)
    thread %>% split(1:nrow(.)) %>%
      walk(function(posting){
        write(paste("<br>#CODE ", paste(
          posting$thread_datum_c, 
          posting$datum_c,
          posting$jahr_c, posting$monat_c, posting$jahr_monat_c, posting$tag_c, posting$wtag_c, posting$stunde_c,
          posting$os_ver_c, posting$maxqda_ver_c, posting$autor_c, posting$typ_c, sep="\\&&")),
          file=cn, append = TRUE)
        write("<br><strong>",file=cn, append = TRUE)
        write(paste(posting$timestamp, posting$autor,"<br>",sep="<br>"), file=cn, append = TRUE)
        write("</strong>",file=cn, append = TRUE)
        write(posting$content, file=cn, append = TRUE)
        write("<br>#ENDCODE", file=cn, append = TRUE)
        write("<br><br>", file=cn, append = TRUE)
      })
  })
write("</body></html>",file=cn, append = TRUE)

close(cn)

forum_postings_1 %>%
  group_by(thread_url) %>%
  summarize(across(.cols=c(Dokumentgruppe:views),
                   .fns=first)) %>%
  mutate(Dokumentname = Dokumentgruppe,
         Dokumentgruppe = "Technikforum") %>%
  arrange(Dokumentname) %>%
  write.xlsx(asTable = T,
             colWidth = "auto",
             file = "thread_vars.xlsx")


